@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import importlib
 import math
 import warnings
 
@@ -8,24 +9,16 @@ from fsspec.core import get_fs_token_paths
 from fsspec.utils import stringify_path
 from packaging.version import parse as parse_version
 
-from ....base import compute_as_if_collection, tokenize
-from ....blockwise import BlockIndex
-from ....delayed import Delayed
-from ....highlevelgraph import HighLevelGraph
-from ....layers import DataFrameIOLayer
-from ....utils import apply, import_required, natural_sort_key, parse_bytes
-from ...core import DataFrame, Scalar, new_dd_object
-from ...methods import concat
-from ..utils import _is_local_fs
-from .utils import Engine, _sort_and_analyze_paths
-
-try:
-    import snappy
-
-    snappy.compress
-except (ImportError, AttributeError):
-    snappy = None
-
+from dask.base import compute_as_if_collection, tokenize
+from dask.blockwise import BlockIndex
+from dask.dataframe.core import DataFrame, Scalar, new_dd_object
+from dask.dataframe.io.parquet.utils import Engine, _sort_and_analyze_paths
+from dask.dataframe.io.utils import _is_local_fs
+from dask.dataframe.methods import concat
+from dask.delayed import Delayed
+from dask.highlevelgraph import HighLevelGraph
+from dask.layers import DataFrameIOLayer
+from dask.utils import apply, import_required, natural_sort_key, parse_bytes
 
 __all__ = ("read_parquet", "to_parquet")
 
@@ -240,16 +233,12 @@ def read_parquet(
         ``"precache_options"`` key. Also, a custom file-open function can be
         used (instead of ``AbstractFileSystem.open``), by specifying the
         desired function under the ``"open_file_func"`` key.
-    engine : str, default 'auto'
-        Parquet reader library to use. Options include: 'auto', 'fastparquet',
-        'pyarrow', 'pyarrow-dataset', and 'pyarrow-legacy'. Defaults to 'auto',
-        which selects the FastParquetEngine if fastparquet is installed (and
-        ArrowDatasetEngine otherwise).  If 'pyarrow' or 'pyarrow-dataset' is
-        specified, the ArrowDatasetEngine (which leverages the pyarrow.dataset
-        API) will be used. If 'pyarrow-legacy' is specified, ArrowLegacyEngine
-        will be used (which leverages the pyarrow.parquet.ParquetDataset API).
-        NOTE: The 'pyarrow-legacy' option (ArrowLegacyEngine) is deprecated
-        for pyarrow>=5.
+    engine : {'auto', 'fastparquet', 'pyarrow'}, default 'auto'
+        Parquet library to use. Options include: 'auto', 'fastparquet', and
+        'pyarrow'. Defaults to 'auto', which uses ``fastparquet`` if it is
+        installed, and falls back to ``pyarrow`` otherwise. Note that in the
+        future this default ordering for 'auto' will switch, with ``pyarrow``
+        being used if it is installed, and falling back to ``fastparquet``.
     gather_statistics : bool, default None
         Gather the statistics for each dataset partition. By default,
         this will only be done if the _metadata file is available. Otherwise,
@@ -368,7 +357,7 @@ def read_parquet(
         columns = list(columns)
 
     if isinstance(engine, str):
-        engine = get_engine(engine)
+        engine = get_engine(engine, bool(kwargs))
 
     if hasattr(path, "name"):
         path = stringify_path(path)
@@ -531,7 +520,7 @@ def to_parquet(
     df,
     path,
     engine="auto",
-    compression="default",
+    compression="snappy",
     write_index=True,
     append=False,
     overwrite=False,
@@ -559,13 +548,15 @@ def to_parquet(
         Destination directory for data.  Prepend with protocol like ``s3://``
         or ``hdfs://`` for remote data.
     engine : {'auto', 'fastparquet', 'pyarrow'}, default 'auto'
-        Parquet library to use. If only one library is installed, it will use
-        that one; if both, it will use 'fastparquet'.
-    compression : string or dict, default 'default'
+        Parquet library to use. Options include: 'auto', 'fastparquet', and
+        'pyarrow'. Defaults to 'auto', which uses ``fastparquet`` if it is
+        installed, and falls back to ``pyarrow`` otherwise. Note that in the
+        future this default ordering for 'auto' will switch, with ``pyarrow``
+        being used if it is installed, and falling back to ``fastparquet``.
+    compression : string or dict, default 'snappy'
         Either a string like ``"snappy"`` or a dictionary mapping column names
-        to compressors like ``{"name": "gzip", "values": "snappy"}``. The
-        default is ``"default"``, which uses the default compression for
-        whichever engine is selected.
+        to compressors like ``{"name": "gzip", "values": "snappy"}``. Defaults
+        to ``"snappy"``.
     write_index : boolean, default True
         Whether or not to write the index. Defaults to True.
     append : bool, default False
@@ -654,10 +645,12 @@ def to_parquet(
     compute_kwargs = compute_kwargs or {}
 
     if compression == "default":
-        if snappy is not None:
-            compression = "snappy"
-        else:
-            compression = None
+        warnings.warn(
+            "compression='default' is deprecated and will be removed in a "
+            "future version, the default for all engines is 'snappy' now.",
+            FutureWarning,
+        )
+        compression = "snappy"
 
     partition_on = partition_on or []
     if isinstance(partition_on, str):
@@ -671,7 +664,7 @@ def to_parquet(
         )
 
     if isinstance(engine, str):
-        engine = get_engine(engine)
+        engine = get_engine(engine, bool(kwargs))
 
     if hasattr(path, "name"):
         path = stringify_path(path)
@@ -819,10 +812,10 @@ def to_parquet(
 
     # Collect metadata and write _metadata.
     # TODO: Use tree-reduction layer (when available)
-    meta_name = "metadata-" + data_write._name
     if write_metadata_file:
+        final_name = "metadata-" + data_write._name
         dsk = {
-            (meta_name, 0): (
+            (final_name, 0): (
                 apply,
                 engine.write_metadata,
                 [
@@ -835,16 +828,22 @@ def to_parquet(
             )
         }
     else:
-        dsk = {(meta_name, 0): (lambda x: None, data_write.__dask_keys__())}
+        # NOTE: We still define a single task to tie everything together
+        # when we are not writing a _metadata file. We do not want to
+        # return `data_write` (or a `data_write.to_bag()`), because calling
+        # `compute()` on a multi-partition collection requires the overhead
+        # of trying to concatenate results on the client.
+        final_name = "store-" + data_write._name
+        dsk = {(final_name, 0): (lambda x: None, data_write.__dask_keys__())}
 
     # Convert data_write + dsk to computable collection
-    graph = HighLevelGraph.from_collections(meta_name, dsk, dependencies=(data_write,))
+    graph = HighLevelGraph.from_collections(final_name, dsk, dependencies=(data_write,))
     if compute:
         return compute_as_if_collection(
-            Scalar, graph, [(meta_name, 0)], **compute_kwargs
+            Scalar, graph, [(final_name, 0)], **compute_kwargs
         )
     else:
-        return Scalar(graph, meta_name, "")
+        return Scalar(graph, final_name, "")
 
 
 def create_metadata_file(
@@ -992,25 +991,19 @@ def create_metadata_file(
 _ENGINES: dict[str, Engine] = {}
 
 
-def get_engine(engine):
+# TODO: remove _warn_engine_default_changing once the default has changed to
+# pyarrow.
+def get_engine(engine, _warn_engine_default_changing=False):
     """Get the parquet engine backend implementation.
 
     Parameters
     ----------
     engine : str, default 'auto'
-        Backend parquet library to use. Options include: 'auto', 'fastparquet',
-        'pyarrow', 'pyarrow-dataset', and 'pyarrow-legacy'. Defaults to 'auto',
-        which selects the FastParquetEngine if fastparquet is installed (and
-        ArrowLegacyEngine otherwise).  If 'pyarrow-dataset' is specified, the
-        ArrowDatasetEngine (which leverages the pyarrow.dataset API) will be used
-        for newer PyArrow versions (>=1.0.0). If 'pyarrow' or 'pyarrow-legacy' are
-        specified, the ArrowLegacyEngine will be used (which leverages the
-        pyarrow.parquet.ParquetDataset API).
-        NOTE: 'pyarrow-dataset' enables row-wise filtering, but requires
-        pyarrow>=1.0. The behavior of 'pyarrow' will most likely change to
-        ArrowDatasetEngine in a future release, and the 'pyarrow-legacy'
-        option will be deprecated once the ParquetDataset API is deprecated.
-    gather_statistics : bool or None (default).
+        Parquet library to use. Options include: 'auto', 'fastparquet', and
+        'pyarrow'. Defaults to 'auto', which uses ``fastparquet`` if it is
+        installed, and falls back to ``pyarrow`` otherwise. Note that in the
+        future this default ordering for 'auto' will switch, with ``pyarrow``
+        being used if it is installed, and falling back to ``fastparquet``.
 
     Returns
     -------
@@ -1020,44 +1013,49 @@ def get_engine(engine):
         return _ENGINES[engine]
 
     if engine == "auto":
-        for eng in ["fastparquet", "pyarrow"]:
-            try:
-                return get_engine(eng)
-            except RuntimeError:
-                pass
-        else:
-            raise RuntimeError("Please install either fastparquet or pyarrow")
+        try:
+            engine = get_engine("fastparquet")
+            if _warn_engine_default_changing and importlib.util.find_spec("pyarrow"):
+                warnings.warn(
+                    "engine='auto' will switch to using pyarrow by default in "
+                    "a future version. To continue using fastparquet even if "
+                    "pyarrow is installed in the future please explicitly "
+                    "specify engine='fastparquet'.",
+                    FutureWarning,
+                )
+            return engine
+        except RuntimeError:
+            pass
+
+        try:
+            return get_engine("pyarrow")
+        except RuntimeError:
+            raise RuntimeError("Please install either fastparquet or pyarrow") from None
 
     elif engine == "fastparquet":
         import_required("fastparquet", "`fastparquet` not installed")
-        from .fastparquet import FastParquetEngine
+        from dask.dataframe.io.parquet.fastparquet import FastParquetEngine
 
         _ENGINES["fastparquet"] = eng = FastParquetEngine
         return eng
 
-    elif engine in ("pyarrow", "arrow", "pyarrow-legacy", "pyarrow-dataset"):
-
+    elif engine in ("pyarrow", "arrow", "pyarrow-dataset"):
         pa = import_required("pyarrow", "`pyarrow` not installed")
         pa_version = parse_version(pa.__version__)
 
-        if engine in ("pyarrow", "arrow"):
-            engine = "pyarrow-dataset"
-        elif pa_version.major >= 5 and engine == "pyarrow-legacy":
-            warnings.warn(
-                "`ArrowLegacyEngine` ('pyarrow-legacy') is deprecated for "
-                "pyarrow>=5 and will be removed in a future release. Please "
-                "use `engine='pyarrow'` or `engine='pyarrow-dataset'`.",
-                FutureWarning,
-            )
+        if engine in ("pyarrow-dataset", "arrow"):
+            engine = "pyarrow"
 
-        if engine == "pyarrow-dataset":
-            from .arrow import ArrowDatasetEngine
+        if engine == "pyarrow":
+            if pa_version.major < 1:
+                raise ImportError(
+                    f"pyarrow-{pa_version.major} does not support the "
+                    f"pyarrow.dataset API. Please install pyarrow>=1."
+                )
+
+            from dask.dataframe.io.parquet.arrow import ArrowDatasetEngine
 
             _ENGINES[engine] = eng = ArrowDatasetEngine
-        else:
-            from .arrow import ArrowLegacyEngine
-
-            _ENGINES[engine] = eng = ArrowLegacyEngine
         return eng
 
     else:
@@ -1163,6 +1161,8 @@ def apply_filters(parts, statistics, filters):
                     if (
                         operator in ("==", "=")
                         and min <= value <= max
+                        or operator == "!="
+                        and (min != value or max != value)
                         or operator == "<"
                         and min < value
                         or operator == "<="
