@@ -4,6 +4,7 @@ distributed = pytest.importorskip("distributed")
 
 import asyncio
 import os
+import sys
 from functools import partial
 from operator import add
 
@@ -25,6 +26,21 @@ if "should_check_state" in get_named_args(gen_cluster):
     cluster = partial(cluster, should_check_state=False)
 
 
+# TODO: the fixture teardown for `cluster_fixture` is failing periodically with
+# a PermissionError on windows only (in CI). Since this fixture lives in the
+# distributed codebase and is nested within other fixtures we use, it's hard to
+# patch it from the dask codebase. And since the error is during fixture
+# teardown, an xfail won't cut it. As a hack, for now we skip all these tests
+# on windows. See https://github.com/dask/dask/issues/8877.
+pytestmark = pytest.mark.skipif(
+    sys.platform == "win32",
+    reason=(
+        "The teardown of distributed.utils_test.cluster_fixture "
+        "fails on windows CI currently"
+    ),
+)
+
+
 def test_can_import_client():
     from dask.distributed import Client  # noqa: F401
 
@@ -343,6 +359,11 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
+    # TODO: this configuration is flaky on osx in CI
+    # See https://github.com/dask/dask/issues/8816
+    if from_futures and sys.platform == "darwin":
+        pytest.xfail("This test sometimes fails on osx in CI")
+
     df = pd.DataFrame({"x": [1, 2, 3] * 5, "y": range(15)})
 
     if from_futures:
